# Make sure shiny, shinydashboard and shinythemes packages are installed. 
library(shiny)
library(shinydashboard)
library(shinythemes)
library(shinyWidgets)
library(coda)                           
library(rjags)
library(tidyverse)
library(manipulate)
library(MCMCvis)

library(paletteer)
library(viridis)
library(viridisLite)
library(ggplot2)
library(RColorBrewer)
library(ggthemes)
 
# Read in the data

HEV.data_Original <- read.csv("HEVTimes_Data.csv")   %>%                                                          # Read HEV Times data from "HeVtimes_Data_260121.CSV" file (Placed in same folder as R script)
                    mutate(N0=10^Starting.value, Nt = 10^(Starting.value - Log.Reduction), Time=Time..Mins.) %>%         # Add columns for N0 (Starting virus), Nt (Virus at time t), and Time in Minutes. 
                    rename(Temperature=Temperature..oC.) %>%                                                             # Renames the temperature column in the Selected data. 
                    filter(EXCLUDE_DATA!="EXCLUDE") %>%                                                                  # Remove any data labeled as "EXCLUDE" in .CSV file.
                    select( Temperature, Time, N0, Nt, Log.Reduction, NewData, EXPERIMENT_DATASET, RUN_NUMBER, Source, EXCLUDE_DATA)  # Select data fields to keep, and drops any not listed here.

 
for (k in 1:length(HEV.data_Original$NewData)){                                                       # Assigns Data origin,  to be "from literature", or from new experiments. 
  if(HEV.data_Original$NewData[k]=="TRUE" ){HEV.data_Original$Data_Origin[k]<- "Experimental" }           #Sets Data Origin to "Experimental" if "Newdata" column is TRUE. 
  else if(HEV.data_Original$NewData[k]=="FALSE")
  {HEV.data_Original$Data_Origin[k]<- "Literature" }    #Sets Data Origin to "Literature", if "Newdata" column entry is FALSE, in .csv data file.  .
  
}  


for (s in 1:length(HEV.data_Original$EXPERIMENT_DATASET)){
  if  ( HEV.data_Original$NewData[s] == TRUE &&  HEV.data_Original$EXPERIMENT_DATASET[s] == "DATASET0"  ){ HEV.data_Original$Single_Points[s] <- "TRUE" }
  
  else if( HEV.data_Original$NewData[s] == TRUE &&  HEV.data_Original$EXPERIMENT_DATASET[s] != "DATASET0"  ){ HEV.data_Original$Single_Points[s] <- "FALSE" }
 
  else if(HEV.data_Original$NewData[s] == FALSE ){ HEV.data_Original$Single_Points[s] <- "Lit" }

}

Temperature_Grouping_Ranges <-   c(0,50,68,75,80,85,90,95,100)                    # Defines start/end points of each temperature group intervals(for plots). Eg 5--67, 68-74, etc. 
Temperature_Group_labels <- c("0-50","50-67", "68-74","75","80","85","90","95","100")  # Define temperature group labels that will appear in plots.

HEV.data_Original$Temp_Group <-   Temperature_Group_labels  [  ( findInterval(  HEV.data_Original$Temperature  , Temperature_Grouping_Ranges ))]
HEV.data_Original$Temp_Group 

RUN_MAX=  max(HEV.data_Original$RUN_NUMBER,  na.rm=T)
  
  
 
Source_options <-  unique( HEV.data_Original$Source )
 

#############################################################################
#########################   DEFINE JAGS SOLVER  FUNCTION #################### 
#############################################################################

 

HEV_JAGS_FUNCTION <- function( Data_In , A_In,  Beta_In,  Temp_fit )  {  
  
   
  A    = A_In
  Beta = Beta_In
  
  data_fit_Temp  <-  Data_In %>%  filter(   Temperature == Temp_fit )
  
  model.inits <- list(A=A_In, Beta=Beta_In)    # KEEP ORDER HERE   E=E_In,
  
  JAGS_data<-list( NLR = - data_fit_Temp$Log.Reduction,
                   Time = data_fit_Temp$Time ,
                   mu1=model.inits[1],
                   mu2=model.inits[2] 
  )
  
  parameters_mod=c("A", "Beta") 
  
  HEV_JAGS_model <- " model {                               # WARNING!!! Lines 83-100 are written using the JAGS language, which uses slightly different syntax than the R language. So, take care if editing these! 
   for(i in 1:length(NLR) ){
   
     NLR[i] ~  dnorm(Mu[i],tau )
     
     Mu[i] <- (-1/2.303)*((  A*Time[i]  )^ Beta )  
     
              }
   
     A ~       dunif( 0 , 2*mu1 )   
    
     Beta ~   dunif( 0 , 2*mu2 )        
                                     # Be careful: dgamma parameters are Shape and rate here (JAGS LANGUAGE FORMAT). 
     
    sigma2 ~    dgamma(1 , 4)        # Higher precision (and smaller variance) means tighter fit. This Increased precision can be achieved by increasing the second parameter in this gamma distribution.
    tau <- 1 / ( sigma2 )
   
   
   } "
  
  #tau <- Tolerance for normal distribution fit - "Case for unknown standard deviation":  HEV.data$Log.Reduction
  #tau <- 1/0.25  # Tau is precision (i.e. 1/ variance)  
  #Mu[i] <- (-1/2.303)*((  A*Time[i]*exp(  (-E)  / (8.314*(Temperature[i]+273.15))) )^ Beta ) 
  # E ~       dunif( 0 , 2*mu2 )  
  
  invisible(capture.output(                # SUPPRESSES OUTPUT FROM PRINTING IN CONSOLE
    
  results <- jags.model(textConnection(HEV_JAGS_model),
                                           data=JAGS_data,
                                                 n.chains = 3)  
                           ))
 
  
  invisible(capture.output(                 # SUPPRESSES OUTPUT FROM PRINTING IN CONSOLE
          
         update(results,1e3)
         
                       ))
  
  
  invisible(capture.output(  
  
  model_sim <- coda.samples(model=results, 
                            variable.names = parameters_mod,
                            n.iter = 5e3)
  ))
  #model_csim<- as.mcmc(do.call(rbind,model_sim))
  
  model_plot<- plot(model_sim)
   
   
 
  #print(summary(model_sim))
  
  Coefficients_Model <- MCMCsummary(model_sim)
  
  A_fitted <- c(Coefficients_Model$mean[1] , Coefficients_Model$`2.5%`[1] , Coefficients_Model$`97.5%`[1],Coefficients_Model$`50%`[1])   
  # A PARAMETER : Store Mean, 2.5 percentile and 97.5 percentile for confidence interval 
  
  Beta_fitted <- c(Coefficients_Model$mean[2] , Coefficients_Model$`2.5%`[2] , Coefficients_Model$`97.5%`[2],Coefficients_Model$`50%`[2])  
  # Beta PARAMETER : Store Mean, 2.5 percentile and 97.5 percentile for confidence interval 
  
  
  
  #Return parameter estimates, and summary stats ... Potential to add additional summary stats too? 
  
  return(list ( Coefficients_Model, A_fitted, Beta_fitted  ) )  #model_sim
  
  
  }

####################################################


############################################################################### 
######################                                 ########################
######################      AP USER INTERFACE (UI)     ######################## 
######################                                 ########################
############################################################################### 



ui <-  dashboardPage(  skin = "green" ,
  
  dashboardHeader(title="HEV Prediction Model"),
  
  
  ##############################################################################
  ########################### SIDEBAR UI         ###############################
  ##############################################################################
  
  dashboardSidebar(sidebarMenu( 
    # Instruction & view data table tab
    menuItem("Select data", tabName = "Select", icon=icon("cogs")),
    
    # tab to make predictions
    menuItem("Filtered data", tabName = "Filtered_data", icon=icon("clipboard")),
    
    # Tab to update the BASIC LEAST SQUARES model
    menuItem("Manual Fit", tabName = "Manual_Fit", icon=icon("tools")),
    
    # Tab to update the BASIC LEAST SQUARES model
    # menuItem("Basic: Least squares Fit", tabName = "Least_Squares", icon=icon("gavel")),
    
    menuItem("Advanced: Bayesian Fit", tabName = "JAGS_Model", icon=icon("bullseye"))
    
    #,
    
    #menuItem("Predict Reduction", tabName = "LR_Predictor", icon=icon("signal"))
     
  )),
  
  
  
  ##############################################################################
  ########################### DASHBOARD BODY UI  ###############################
  ##############################################################################
  
  
  
  dashboardBody(
    tabItems(
      
      
      
      ##############################################################################
      ########################### SELECT DATA TAB UI  ##############################
      ##############################################################################
      
      
      tabItem(tabName="Select",
              
             
             
             box(title = "Selected data:", width=12, solidHeader = TRUE, status = "primary",
                 plotOutput("Selected_Data_plot", 
                            brush = brushOpts(id = "Selected_Data_plot_brush", resetOnNew = TRUE),
                            height = 600) ,
                 
                 downloadButton("downloadSelectedDataPlot", label = "Download Plot") ),
             
             
             fluidRow(   box( title = "Filter selected data by:",  width=12, solidHeader = TRUE, 
                              status = "primary",   
                              
                              column ( width=6, 
              
             sliderInput( "RangeLogReduction" , "Range Log reduction:", min = -0.2, max = 6, value = c(0,4)), 
             sliderInput( "Timerange" , "Time range (minutes):", min = 0, max = 300, value = c(0,22)) ,
    #To plot Soil          sliderInput( "Timerange" , "Time range (minutes):", min = 0, max = 101000, value = c(0,22)) ,
             sliderInput( "Trange" , "Temperature range (Celcius):", min = 0, max = 150, value = c(50,100)),
             sliderInput( "Y_Axis_range" , "Y-Axis adjust:", min =  -6, max = 0.2, value = c(-4,0.2)) 
             ),
             column( width = 6,
                     
                     p(style="text-align: justify;", strong("NOTE:"),("To calculate unique model parameters for an individual viral source, only that ONE 
                                        source should be selected here.")
                     ),
             
             checkboxGroupInput("Filter_Source", "Filter by source:", Source_options, selected=Source_options) ,
             checkboxGroupInput("Single_point_vs_Time_series_GCU_data_Filter", "GCU Experimental/Literature Data:", choiceNames = c("GCU Experiments: Single Data Points", "GCU Experiments :Full Time Series", "Literature Data"), choiceValues = c("TRUE","FALSE","Lit"),selected = c("TRUE", "FALSE","Lit") ) 
             ,
             # Button
             downloadButton("downloadSelectedData", "Download Selected Data (.csv)") 
             
           
             )
              
             )
             
                       )   
            
             
            
               
                      
               
  ),
      
  
  
  
  
  ##############################################################################
  ########################### FILTER DATA TAB UI  ##############################
  ##############################################################################
  
      tabItem(tabName="Filtered_data",
             
              
             fluidRow(box( 
               title = "Filtered Data:",  width=12,   solidHeader = TRUE, status = "primary", 
               dataTableOutput("HEV_data_Table") 
               
               ))  
              
              
      ),

  
  ##############################################################################
  ########################### MANUAL FIT TAB UI  ##############################
  ##############################################################################
       
      tabItem(tabName="Manual_Fit",
              
              
              fluidRow(box( 
                 
                title = "Model Equation:", width=12,   solidHeader = TRUE, status = "primary" ,
                withMathJax(  "       
                
             $$ - || \\mbox{Log Reduction } || =   \\frac{-1}{2.303} \\cdot  
                
                \\Bigl (  A t  \\Bigr)^{\\beta}   $$ "),
          
 
                
              ))  ,
 
              box(title = "Manual Fit:", width=12, solidHeader = TRUE, status = "primary",
                  plotOutput("Manual_Fit_Plot",   
                             brush = brushOpts(id = "Manual_Fit_Plot_brush", resetOnNew = TRUE),
                             height = 600) ,
                  
                  downloadButton("downloadManualFitPlot", label = "Download Manual Fit Plot")  ),
              
              
              fluidRow(box( 
                title = "Manual Fit Curve Settings:",  width=12,   solidHeader = TRUE, status = "primary"  ,
                
                column ( width=6, 
                         withMathJax(),
                sliderInput( "A_manual" , "\\(A\\)  parameter:", min = 0, max = 2, value =  0.1, step = 0.0005) 
                       ),
                
                column ( width=6, 
              
                sliderInput( "beta_manual" , "\\(\\beta\\)  parameter:", min = 0.001, max =5, value =  1.5, step = 0.05) 
                        )
                
                
              ))
              
              
              
      ), 
      
  
 
  
  
  ##############################################################################
  ########################### JAGS MODEL  TAB UI  ##############################
  ##############################################################################
      #JAGS model tab
      tabItem(tabName="JAGS_Model",
              title = "Bayesian (JAGS) Model:",
              
              fluidRow( column( width=12 , align = "center",
                                p(style="text-align: justify;",strong("NOTE:"),("Clicking 'Estimate Model Parameters!' 
                                          fits a single pair of parameters, to ALL selected data points, 
                                          that have temperature exactly equal the temperature selected below."),
                                  strong("To calculate unique parameters for an individual source, only that ONE 
                                        source should be selected on the 'select data' tab.") )
              )),
              
              fluidRow(box( 
                title = "Parameter value estimates (prior means):",  width=12,   solidHeader = TRUE, status = "primary",  
               
                 withMathJax(),
                
                column(width=3, 
                       
                       numericInput("A_Prior_guess",label = "\\(A\\) prior mean, \\(\\mu_{A}\\):",    value = 0.1),
                        
                       
                       actionButton("simulate_JAGS", "Estimate Model Parameters!")
  
                ),
                column( width=3,  
                        
                        numericInput( "Beta_Prior_guess", label = "\\(\\beta\\) prior mean, \\(\\mu_{\\beta}\\):",  value = 1.5) 
                ),
                
                column(width=6, 
     
                       uiOutput("slider_Temp_to_fit") ,
                       p(strong("NOTE:"),("Fitted parameters are unique to the temperature selected here."))
                     
                      ),
                ),   # end of box
                
 
              ),
              
              fluidRow( column( width=12 , align = "center",
                                p((strong("Prior distributions:")),(" \\(A\\) ~ Uniform( 0 , 2 \\(\\mu_{A}\\) );      
                                                                    and   \\(\\beta\\) ~ Uniform( 0 , 2 \\(\\mu_{\\beta}\\) ).     ")  
              ))),
              
              
              fluidRow(box( 
                title = "Summary Statistics:",  width=12,   solidHeader = TRUE, status = "primary", 
                 
                verbatimTextOutput("MCMC_Results_Table_1") ,
                
                
                downloadButton("downloadMCMCSummary", "Download MCMC Summary (.csv)") 
 
                
              ))  ,
              
              box( 
                title = "Simulation Results",  width=12,   solidHeader = TRUE, status = "primary", 
                
               
                plotOutput("JAGS_FIT_RESULTS_HEVTIMES",
                           brush = brushOpts(id = "JAGS_FIT_RESULTS_HEVTIMES", resetOnNew = TRUE),
                           height = 600) ,
                
             
              
              
              
              column (    width = 6 ,
                          downloadButton("downloadJAGS_FIT_RESULTS_HEVTIMES", label = "Download HEVTimes Results") 
              ),
              
              column( width = 6, 
                      
                      p("( 95% Confidence interval: [\\( Q_{025}\\), \\(Q_{975}\\)])")   )
              
              
              )  
              
          
      ) 
  #,
  
 #  tabItem(tabName="LR_Predictor",
          
          
          
  #        box(title = "Predicted Log Reduction:", width=12, solidHeader = TRUE, status = "primary"
               
              
              
   #            ),
          
          
   #       fluidRow(   box( title = "Scenario to predict:",  width=12, solidHeader = TRUE, 
   #                        status = "primary",    column ( width=6, 
                                                           
   #                     sliderInput( "Predictor_Temperature" , "Temperature:", min = 0, max = 200, value = 100)  
                                                           
                                                            
                                                           
   #                        ),
   #                        column( width = 6,
    #                               sliderInput( "Predictor_Time" , "Time (minutes):", min = 0, max = 200, value = 20 )  
                                    
   #                        )
                           
 # )
#  )   
#  ) 
      
      
    )
  )
)

 
server <- function(input, output, session ) {
  

 
  
  
 
 
  # #########################################################
  # #######   INTERACTIVE FILTER: SELECT DATA    ############
  # ######################################################### 
   
  
  HEV.data  <- reactive({ #If no sources are selected
    if (is.null(input$Filter_Source)) {
      return(NULL)
       }    
    
    HEV.data_Original %>%
      filter(
               Log.Reduction >= input$RangeLogReduction[1] ,
               Log.Reduction <= input$RangeLogReduction[2] ,
               Time >=input$Timerange[1],
               Time <=input$Timerange[2],
               Temperature >= input$Trange[1] ,  
               Temperature <= input$Trange[2] ,
               Source %in% input$Filter_Source  ,
               Single_Points   %in%   input$Single_point_vs_Time_series_GCU_data_Filter
               )
     
  })
  
  
  
  
  # #########################################################
  # #######   Reactive Temperature slider:           ######## 
  #########   AUTO ADJUST TO SELECT DATA             ######## 
  #########  THAT HAVE MORE THAN ONE DATA POINT      ######## 
  # ######################################################### 
   
   output$slider_Temp_to_fit <- renderUI({ 
     
     HEV.data_duplicate_Temps <-     duplicated( HEV.data()$Temperature )
     Temperature_Options_test <- sort( unique(   HEV.data()$Temperature[   HEV.data_duplicate_Temps]   ))
     
       
      shinyWidgets::sliderTextInput(inputId = "Temp_fit_chosen", 
                                   label = " Temperature to fit (Degrees C):", 
                                   choices = Temperature_Options_test,
                                   selected = max(Temperature_Options_test ) )   #set value to max choice
    
    
    
  })
  
  
 
  ##########################################################
  ############      DOWNLOAD csv : SELECTED DATA  ##########
  ########################################################## 
   
  output$downloadSelectedData <- downloadHandler(
    filename = function() {
      paste('Selected_Data_HEVTimes_App', '.csv', sep = '')
    },
    content = function(file) {
      write.csv( HEV.data() , file, row.names = FALSE)
    }
  )  
  
  
  
  ##########################################################
  ############      TABLE: SELECTED DATA    ################
  ########################################################## 
    
  output$HEV_data_Table <-  renderDataTable(    HEV.data() %>% select( Temperature, Time, N0, Nt, Log.Reduction, Source, Data_Origin, NewData  )  )        
  
  
 
  
  ##########################################################
  ############     PLOT:   SELECTED DATA    ################
  ##########################################################   
  
  output$Selected_Data_plot <-  renderPlot({
      
    
    
    ggplot (  HEV.data(), aes ( x=  Time, y = - Log.Reduction , group = RUN_NUMBER   )   )  +
               
              facet_grid(vars(Source) )+         # Splits into 3 plots by source
       
              geom_point( aes ( shape = Data_Origin, 
                                fill  = factor(Temp_Group,levels =Temperature_Group_labels) ) ,
                                colour="black",
                                size=3) +      # factor orders the legend entries
   
              scale_shape_manual(values = c(21, 24, 22))+
              scale_fill_manual(values = rev( RColorBrewer::brewer.pal(8,'Spectral'))) +
              scale_colour_manual( values = rev( RColorBrewer::brewer.pal(8,'Spectral'))) +
             
              geom_line(data =HEV.data() %>% filter(RUN_NUMBER %in%  seq(1,RUN_MAX,1 )),
                           aes(group = RUN_NUMBER,color=  factor(Temp_Group, levels = Temperature_Group_labels)  ) )+
      
      
          
             guides(fill = guide_legend(order = 1,override.aes = list(shape = 21)),   # specify order or legends appear in. 
                   shape = guide_legend(order = 2, override.aes = list(fill = 1)),
                   colour=FALSE)+
             labs(fill = "Temperature, \u00B0C", shape = "Data Origin")+
       
            xlim(input$Timerange[1],input$Timerange[2])+
            ylim(input$Y_Axis_range[1], input$Y_Axis_range[2])+
      
             ylab("Log Change")+      
             xlab("Time (min)") 
  }, res=96)
  
  
  output$downloadSelectedDataPlot <- downloadHandler(
    filename = function(){
      paste('Selected_Data_HEVTimes', '.png', sep = '')
    },
    content = function(file){
       Selected_Data_plot_on_server <- 
         
         
         ggplot (  HEV.data(), aes ( x=  Time, y = - Log.Reduction , group = RUN_NUMBER   )   )  +
         
         facet_grid(vars(Source) )+         # Splits into 3 plots by source
         
         geom_point( aes ( shape = Data_Origin, 
                           fill  = factor(Temp_Group,levels =Temperature_Group_labels) ) ,
                     colour="black",
                     size=3) +      # factor orders the legend entries
         
         scale_shape_manual(values = c(21, 24, 22))+
         scale_fill_manual(values = rev( RColorBrewer::brewer.pal(8,'Spectral'))) +
         scale_colour_manual( values = rev( RColorBrewer::brewer.pal(8,'Spectral'))) +
         
         geom_line(data =HEV.data() %>% filter(RUN_NUMBER %in%  seq(1,RUN_MAX,1 )),
                   aes(group = RUN_NUMBER,color=  factor(Temp_Group, levels = Temperature_Group_labels)  ) )+
         
         
         
         guides(fill = guide_legend(order = 1,override.aes = list(shape = 21)),   # specify order or legends appear in. 
                shape = guide_legend(order = 2, override.aes = list(fill = 1)),
                colour=FALSE)+
         labs(fill = "Temperature, \u00B0C", shape = "Data Origin")+
         
         xlim(input$Timerange[1],input$Timerange[2])+
         ylim(input$Y_Axis_range[1], input$Y_Axis_range[2])+
         ylab("Log Change")+ 
         xlab("Time (min)") 
       
         
         
      ggsave(file, plot = Selected_Data_plot_on_server , device = 'png', height=6, width=6, units="in", dpi=500)
    }
  )
  
  
 
   
  ##########################################################
  ###############    PLOT:    MANUAL FIT    ################
  ##########################################################
  
 
  output$Manual_Fit_Plot <-  renderPlot({
    
    ggplot (  HEV.data(), aes ( x=  Time, y = - Log.Reduction , group = RUN_NUMBER   )   )  +
      
      facet_grid(vars(Source) )+         # Splits into 3 plots by source
      
      geom_point( aes ( shape = Data_Origin, 
                        fill  = factor(Temp_Group,levels =Temperature_Group_labels) ) ,
                  colour="black",
                  size=3) +      # factor orders the legend entries
      
      scale_shape_manual(values = c(21, 24, 22))+
      scale_fill_manual(values = rev( RColorBrewer::brewer.pal(8,'Spectral'))) +
      scale_colour_manual( values = rev( RColorBrewer::brewer.pal(8,'Spectral'))) +
      
      geom_line(data =HEV.data() %>% filter(RUN_NUMBER %in%  seq(1,RUN_MAX,1 )),
                aes(group = RUN_NUMBER,color=  factor(Temp_Group, levels = Temperature_Group_labels)  ) )+
      
      
      
      guides(fill = guide_legend(order = 1,override.aes = list(shape = 21)),   # specify order or legends appear in. 
             shape = guide_legend(order = 2, override.aes = list(fill = 1)),
             colour=FALSE)+
      labs(fill = "Temperature, \u00B0C", shape = "Data Origin")+
      
      xlim(input$Timerange[1],input$Timerange[2])+
      coord_cartesian(ylim=c(input$Y_Axis_range[1], input$Y_Axis_range[2]))+
 
      stat_function(fun = function(.x)    (-1/2.303)*((   input$A_manual *.x )^ input$beta_manual  )     ,
                         aes(group = 1  ))  +
      
      ylab("Log Change")+ 
      xlab("Time (min)")
         
                                                              }, res=96)
   
   
   output$downloadManualFitPlot <- downloadHandler(
     filename = function(){
       paste('Manual_Fit_HEVTimes','.png', sep = '')
     },
     content = function(file){
       
         Manual_Fit_Plot_on_server <- ggplot (  HEV.data(), aes ( x=  Time, y = - Log.Reduction , group = RUN_NUMBER   )   )  +
         
         facet_grid(vars(Source) )+         # Splits into 3 plots by source
         
         geom_point( aes ( shape = Data_Origin, 
                           fill  = factor(Temp_Group,levels =Temperature_Group_labels) ) ,
                     colour="black",
                     size=3) +      # factor orders the legend entries
         
         scale_shape_manual(values = c(21, 24, 22))+
         scale_fill_manual(values = rev( RColorBrewer::brewer.pal(8,'Spectral'))) +
         scale_colour_manual( values = rev( RColorBrewer::brewer.pal(8,'Spectral'))) +
         
         geom_line(data =HEV.data() %>% filter(RUN_NUMBER %in%  seq(1,RUN_MAX,1 )),
                   aes(group = RUN_NUMBER,color=  factor(Temp_Group, levels = Temperature_Group_labels)  ) )+
         
         
         
         guides(fill = guide_legend(order = 1,override.aes = list(shape = 21)),   # specify order or legends appear in. 
                shape = guide_legend(order = 2, override.aes = list(fill = 1)),
                colour=FALSE)+
           labs(fill = "Temperature, \u00B0C", shape = "Data Origin")+
         
         xlim(input$Timerange[1],input$Timerange[2])+
         coord_cartesian(ylim=c(input$Y_Axis_range[1], input$Y_Axis_range[2])) +
           
           stat_function(fun = function(.x)    (-1/2.303)*((   input$A_manual *.x )^ input$beta_manual  )     ,
                         aes(group = 1 ))  +
         ylab("Log Change")+ 
           xlab("Time (min)")
         
       ggsave(file, plot = Manual_Fit_Plot_on_server , device = 'png', height=6, width=6, units="in", dpi=500)
     }
   )
   
   

 
   
   
   
   ##########################################################
   ######  START: BAYESIAN (JAGS) MCMC MODEL   ############ 
   ########################################################## 
   

   
   A_prior <- eventReactive(input$simulate_JAGS , {
     
                     input$A_Prior_guess 
                           })
   
   
   beta_prior <- eventReactive(input$simulate_JAGS , {
      
                    input$Beta_Prior_guess
                             })
    
   Temp_fit <- eventReactive(input$simulate_JAGS , {
      
                     input$Temp_fit_chosen
                              })
   
 
   
   observeEvent(input$simulate_JAGS, {
     message("Performing Baysian (JAGS) simulation... (Calculations may take some time to run).")
   })
   
 
   observeEvent(input$A_manual, {
     updateNumericInput(session, "A_Prior_guess", value = input$A_manual) })
   observeEvent(input$beta_manual, {
     updateNumericInput(session, "Beta_Prior_guess", value = input$beta_manual) })
   
   
    
   
    HEV_Times_JAGS_Simulation_Results <- eventReactive(input$simulate_JAGS , {HEV_JAGS_FUNCTION(HEV.data(), 
                                                                    A_prior(),  
                                                                    beta_prior(), 
                                                                    Temp_fit())  
                                                  } )
                 
 
    output$MCMC_Results_Table_1 <-   renderPrint({ 
      
                            list ( HEV_Times_JAGS_Simulation_Results()[[1]]   )
        } )
   
    
      
    output$JAGS_FIT_RESULTS_HEVTIMES <- renderPlot({    if (is.null(HEV_Times_JAGS_Simulation_Results()[[2]][1])) return()
            
      
                                               
        ggplot (  HEV.data(), aes ( x=  Time, y = - Log.Reduction    )   )  +
        
        facet_grid(vars(Source) )+         # Splits into 3 plots by source
        
        geom_point( aes ( shape = Data_Origin, 
                          fill  = factor(Temp_Group,levels =Temperature_Group_labels) ) ,
                    colour="black",
                    size=3) +      # factor orders the legend entries
        
          scale_shape_manual(values = c(21, 24, 22))+
          scale_fill_manual(values = rev( RColorBrewer::brewer.pal(8,'Spectral'))) +
          scale_colour_manual( values = c("black","blue","red")) +
          
          guides(fill = guide_legend(order = 1,override.aes = list(shape = 21)),   # specify order or legends appear in. 
                 shape = guide_legend(order = 2, override.aes = list(fill = 1)),
                 colour=guide_legend(order = 3 , override.aes = list(linetype = c(1,2,2) ))) +
          labs(fill = "Temperature, \u00B0C", shape = "Data Origin", colour ="Simulation Results")+  
        
           
          # PLOTS EQUATION USING MEAN PARAMETER VALUES
      #    stat_function(fun = function(.x) 
      #      (-1/2.303)*((    HEV_Times_JAGS_Simulation_Results()[[2]][1]  *.x )^ HEV_Times_JAGS_Simulation_Results()[[3]][1]    ),  # Mean line
      #      colour="red" ) +
          
          # PLOTS EQUATION USING MEDIAN PARAMETER VALUES
          stat_function(fun = function(.x) 
            (-1/2.303)*((    HEV_Times_JAGS_Simulation_Results()[[2]][4]  *.x )^ HEV_Times_JAGS_Simulation_Results()[[3]][4]    ),  # Median line
            aes(colour="Fitted Model") ) +
          
      
          
          # PLOTS EQUATION USING 97.5% quantile PARAMETER VALUES
          stat_function(fun = function(.x) 
           { pmin(((-1/2.303)*((    HEV_Times_JAGS_Simulation_Results()[[2]][3]  *.x )^ HEV_Times_JAGS_Simulation_Results()[[3]][3]    )),
                 ((-1/2.303)*((    HEV_Times_JAGS_Simulation_Results()[[2]][2]  *.x )^ HEV_Times_JAGS_Simulation_Results()[[3]][2]    )),
                 ((-1/2.303)*((    HEV_Times_JAGS_Simulation_Results()[[2]][2]  *.x )^ HEV_Times_JAGS_Simulation_Results()[[3]][3]    )),
                 ((-1/2.303)*((    HEV_Times_JAGS_Simulation_Results()[[2]][3]  *.x )^ HEV_Times_JAGS_Simulation_Results()[[3]][2]    ))
                 )},   # 97.5% line
           aes(colour="Fitted Q_025"), linetype="dashed") +
        
          # PLOTS EQUATION USING 2.5% quantile PARAMETER VALUES
          stat_function(fun = function(.x) 
          { pmax(((-1/2.303)*((    HEV_Times_JAGS_Simulation_Results()[[2]][2]  *.x )^ HEV_Times_JAGS_Simulation_Results()[[3]][2]    )),
                 ((-1/2.303)*((    HEV_Times_JAGS_Simulation_Results()[[2]][3]  *.x )^ HEV_Times_JAGS_Simulation_Results()[[3]][3]    )),
                 ((-1/2.303)*((    HEV_Times_JAGS_Simulation_Results()[[2]][2]  *.x )^ HEV_Times_JAGS_Simulation_Results()[[3]][3]    )),
                 ((-1/2.303)*((    HEV_Times_JAGS_Simulation_Results()[[2]][3]  *.x )^ HEV_Times_JAGS_Simulation_Results()[[3]][2]    )))},   # 2.5% line
          aes(colour="Fitted Q_975"), linetype="dashed" ) +
          
          ylab("Log Change")+ 
          xlab("Time (min)")+
          
          xlim(input$Timerange[1],input$Timerange[2])+
          coord_cartesian(ylim=c(input$Y_Axis_range[1], input$Y_Axis_range[2])) +
          
          ggtitle(   paste("Temperature = ",Temp_fit(),"  \u00B0C ;  A = ",round(HEV_Times_JAGS_Simulation_Results()[[2]][4], digits = 3), "; Beta =" , round( HEV_Times_JAGS_Simulation_Results()[[3]][4], digits=3), sep = "") )   
        
        } )
   
 
    output$downloadJAGS_FIT_RESULTS_HEVTIMES <- downloadHandler(
      filename = function(){
        paste('HEVTIMES_Model_Results','.png',sep ='')
      },
      content = function(file){   
         
        JAGS_SimulationPlot_Server <-  ggplot (  HEV.data(), aes ( x=  Time, y = - Log.Reduction    )   )  +
          
          facet_grid(vars(Source) )+         # Splits into 3 plots by source
          
          geom_point( aes ( shape = Data_Origin, 
                            fill  = factor(Temp_Group,levels =Temperature_Group_labels) ) ,
                      colour="black",
                      size=3) +      # factor orders the legend entries
          
          scale_shape_manual(values = c(21, 24, 22))+
          scale_fill_manual(values = rev( RColorBrewer::brewer.pal(8,'Spectral'))) +
          scale_colour_manual( values = c("black","blue","red")) +
          
          guides(fill = guide_legend(order = 1,override.aes = list(shape = 21)),   # specify order or legends appear in. 
                 shape = guide_legend(order = 2, override.aes = list(fill = 1)),
                 colour=guide_legend(order = 3 , override.aes = list(linetype = c(1,2,2) ))) +
          labs(fill = "Temperature, \u00B0C", shape = "Data Origin", colour ="Simulation Results")+  
          
          
          # PLOTS EQUATION USING MEAN PARAMETER VALUES
          #    stat_function(fun = function(.x) 
          #      (-1/2.303)*((    HEV_Times_JAGS_Simulation_Results()[[2]][1]  *.x )^ HEV_Times_JAGS_Simulation_Results()[[3]][1]    ),  # Mean line
          #      colour="red" ) +
          
          # PLOTS EQUATION USING MEDIAN PARAMETER VALUES
          stat_function(fun = function(.x) 
            (-1/2.303)*((    HEV_Times_JAGS_Simulation_Results()[[2]][4]  *.x )^ HEV_Times_JAGS_Simulation_Results()[[3]][4]    ),  # Median line
            aes(colour="Fitted Model") ) +
          
          
          
          # PLOTS EQUATION USING 97.5% quantile PARAMETER VALUES
          stat_function(fun = function(.x) 
          { pmin(((-1/2.303)*((    HEV_Times_JAGS_Simulation_Results()[[2]][3]  *.x )^ HEV_Times_JAGS_Simulation_Results()[[3]][3]    )),
                 ((-1/2.303)*((    HEV_Times_JAGS_Simulation_Results()[[2]][2]  *.x )^ HEV_Times_JAGS_Simulation_Results()[[3]][2]    )),
                 ((-1/2.303)*((    HEV_Times_JAGS_Simulation_Results()[[2]][2]  *.x )^ HEV_Times_JAGS_Simulation_Results()[[3]][3]    )),
                 ((-1/2.303)*((    HEV_Times_JAGS_Simulation_Results()[[2]][3]  *.x )^ HEV_Times_JAGS_Simulation_Results()[[3]][2]    ))
          )},   # 97.5% line
          aes(colour="Fitted Q_025"), linetype="dashed") +
          
          # PLOTS EQUATION USING 2.5% quantile PARAMETER VALUES
          stat_function(fun = function(.x) 
          { pmax(((-1/2.303)*((    HEV_Times_JAGS_Simulation_Results()[[2]][2]  *.x )^ HEV_Times_JAGS_Simulation_Results()[[3]][2]    )),
                 ((-1/2.303)*((    HEV_Times_JAGS_Simulation_Results()[[2]][3]  *.x )^ HEV_Times_JAGS_Simulation_Results()[[3]][3]    )),
                 ((-1/2.303)*((    HEV_Times_JAGS_Simulation_Results()[[2]][2]  *.x )^ HEV_Times_JAGS_Simulation_Results()[[3]][3]    )),
                 ((-1/2.303)*((    HEV_Times_JAGS_Simulation_Results()[[2]][3]  *.x )^ HEV_Times_JAGS_Simulation_Results()[[3]][2]    )))},   # 2.5% line
          aes(colour="Fitted Q_975"), linetype="dashed" ) +
          
          ylab("Log Change")+ 
          xlab("Time (min)")+
          
          xlim(input$Timerange[1],input$Timerange[2])+
          coord_cartesian(ylim=c(input$Y_Axis_range[1], input$Y_Axis_range[2])) +
          
          ggtitle(   paste("Temperature = ",Temp_fit(),"  \u00B0C ;  A = ",round(HEV_Times_JAGS_Simulation_Results()[[2]][4], digits = 3), "; Beta =" , round( HEV_Times_JAGS_Simulation_Results()[[3]][4], digits=3), sep = "") )   
        

        ggsave(file,plot=JAGS_SimulationPlot_Server,device='png', height=6, width=6, units="in", dpi=500)
      }
    )
    
    
    
    
    ##########################################################
    ############      DOWNLOAD csv : SELECTED DATA  ##########
    ########################################################## 
    
    output$downloadMCMCSummary <- downloadHandler(
      filename = function() {
        paste('MCMC_Summary_HEVTimes_App', '.csv', sep = '')
      },
      content = function(file) {
        write.csv( HEV_Times_JAGS_Simulation_Results()[[1]] , file, row.names = TRUE)
      }
    )  
    
   ##########################################################
   ######  END: BAYESIAN (JAGS) MCMC MODEL   ############ 
   ########################################################## 
 
   
 
}

shinyApp(ui, server)
